'==============
'SOUNDBALLS.BAS
'==============
'A sound immersion environmet.
'Coded by Dav, AUG/2024 in QB64PE v3.13

'Enter a world where balls have their own sound.
'By placing the balls in desired poisitions you
'can create your own unique sound environment.

'========
'CONTROLS
'========

'Move the balls to place its unique sound in the environment.
'Move ball left to place its sound more left in the stereo field.
'Move ball right to place its sound more right in the stereo field.
'Move the balls upward in the distace to make it get softer.
'Move the balls down closer towards you to make it get louder.

'Right click on a ball to turn its sound On/Off.
'Ball will glow when it's turn on, doesn't when off.

'Press any key to leave the soundball world.

'=========================================================
' * USE HEADPHONES OR EXTERNAL SPEAKERS FOR BEST RESULTS *
'=========================================================

Screen _NewImage(1000, 700, 32)
_FullScreen _SquarePixels

Const balls = 8

Dim ballX(balls) As Integer
Dim ballY(balls) As Integer
Dim ballr(balls), ballg(balls), ballb(balls), balla(balls)
Dim ballRadius(balls) As Integer
Dim ballonoff(balls)
Dim ballsound&(balls)
Dim isDragging As Integer
Dim selectedBall As Integer

'load all sound files
For s = 1 To 8
    nam$ = "sound" + LTrim$(RTrim$(Str$(s))) + ".ogg"
    ballsound&(s) = _SndOpen(nam$, "stream")
    If ballsound&(s) > 0 Then
        _SndVol ballsound&(s), 0 'defaunt vol muted
        _SndLoop ballsound&(s) 'start all sounds so the sync together
    Else
        Print "Failed to load sound file "; nam$
        End
    End If
Next

' Initialize balls
For i = 1 To balls
    ballX(i) = Int(Rnd * (_Width - 100)) + 50
    ballY(i) = Int(Rnd * (_Height - 100)) + 50
    If ballY(i) < 100 Then ballY(i) = 100
    ballr(i) = 200 + (Rnd * 55)
    ballg(i) = 200 + (Rnd * 55)
    ballb(i) = 200 + (Rnd * 55)
    ballRadius(i) = ballY(i) / 7
    ballonoff(i) = 0 'all balls off
    If ballonoff(i) = 0 Then
        balla(i) = 55
    Else
        balla(i) = 155
    End If
Next i

'Turn on a ball at the beginning to start things off
ballonoff(8) = 1: balla(8) = 155


'=== draw background images to use

'create star image for scrolling
star& = _NewImage(1000, 100, 32)
_Dest star&
Cls , _RGB(0, 0, 0) 'black sky
'stars
For s = 1 To 100
    Line (Rnd * _Width, Rnd * 100)-Step(Rnd * 2, Rnd * 2), _RGBA(255, 255, 255, 100 + Rnd * 100)
Next

_Dest 0

'draw green gradient ground
c = 0
For y = 100 To _Height
    Line (0, y)-(_Width, y), _RGBA(0, c, c / 2, 225 + Rnd * 25), BF
    c = c + .5: If c = 128 Then c = 128
Next

'info at bottom
_PrintMode _KeepBackground
Color _RGBA(0, 0, 0, 96)
_PrintString (20, 675), "Right click turns ball On/Off"
_PrintString (375, 675), "Move Up/Down changes volume"
_PrintString (715, 675), "Move Left/Right changes placement"

'grab this as main background image
back& = _CopyImage(_Display)

'====================================

'flags main loop uses
isDragging = 0
selectedBall = 0


Do

    _PutImage (0, 0)-(_Width, _Height), back&

    'scroll stars
    _PutImage (starx, 0), star&
    _PutImage (starx + _Width, 0), star&
    starx = starx - .5: If starx <= -_Width Then starx = 0

    While _MouseInput: Wend

    'if the left mouse button pressed
    If _MouseButton(1) And _MouseY > 100 Then
        'if not dragging a ball yet
        If isDragging = 0 Then
            For i = 1 To balls
                'if clicked mouse on a ball
                If (Sqr((_MouseX - ballX(i)) ^ 2 + (_MouseY - ballY(i)) ^ 2) < ballRadius(i)) Then
                    selectedBall = i 'Select ball
                    isDragging = 1 'dragging flag on
                    Exit For
                End If
            Next i
        End If

        'If dragging flag on, move selected ball
        If isDragging = 1 And selectedBall > 0 Then
            ballX(selectedBall) = _MouseX
            ballY(selectedBall) = _MouseY
        End If
    Else
        'left mouse button released, so reset dragging flag
        isDragging = 0
        selectedBall = 0
    End If

    'see if right click on a ball
    If _MouseButton(2) And _MouseY > 100 Then
        For i = 1 To balls
            'if right clicked mouse on a ball
            If (Sqr((_MouseX - ballX(i)) ^ 2 + (_MouseY - ballY(i)) ^ 2) < ballRadius(i)) Then
                If ballonoff(i) = 1 Then
                    ballonoff(i) = 0
                    balla(i) = 55
                    _SndVol ballsound&(i), 0 'mute volume
                Else
                    ballonoff(i) = 1
                    balla(i) = 155
                    '_SndLoop ballsound&(i)
                End If
                _Delay .15
                Exit For
            End If
        Next i
    End If


    'change balls radius based on the ball's y position
    For i = 1 To balls
        ballRadius(i) = ballY(i) / 7
        'make sure radius don't go outside these...
        If ballRadius(i) < 5 Then ballRadius(i) = 5
        If ballRadius(i) > 150 Then ballRadius(i) = 150
    Next i

    'Draw every ball
    For i = 1 To balls
        glowball ballX(i), ballY(i), ballRadius(i), ballr(i), ballg(i), ballb(i), balla(i) + ballRadius(i)
        If ballonoff(i) = 0 Then
            Circle (ballX(i), ballY(i)), ballRadius(i), _RGB(64, 96, 64)
        Else
            Circle (ballX(i), ballY(i)), ballRadius(i), _RGB(0, Rnd * 255, 0)
        End If
    Next i

    'adjust sound volume and balance for each ball
    For i = 1 To balls
        'if ball is turned on
        If ballonoff(i) = 1 Then
            _SndVol ballsound&(i), (ballRadius(i) - 15) / 100
            balance = Int(((ballX(i) - _Width / 2) / (_Width / 2)) * 100)
            _SndBal ballsound&(i), (balance / 100)
        Else
            _SndVol ballsound&(i), 0
        End If
    Next

    _Display
    _Limit 30

Loop Until InKey$ <> ""

End



Sub glowball (x, y, size, r, g, b, a)
    t = Timer
    For y2 = y - size To y + size
        For x2 = x - size To x + size
            If Sqr((x2 - x) ^ 2 + (y2 - y) ^ 2) <= size Then
                clr = (size - (Sqr((x2 - x) * (x2 - x) + (y2 - y) * (y2 - y)))) / size
                noise = Int(Rnd * 50)
                r = Sin(6.005 * t) * size - y2 + size + 255 * 2
                g = Sin(3.001 * t) * size - x2 + size + 255
                b = Sin(2.001 * x2 / size + t + y2 / size) * r + 255
                t = t + .00195
                PSet (x2, y2), _RGBA(clr * r - noise, clr * g - noise, clr * b - noise, a)
            End If
        Next
    Next

End Sub

